/**
 * Copyright 1998-2007, CHISEL Group, University of Victoria, Victoria, BC, Canada.
 * All rights reserved.
 */
package ca.uvic.csr.shrimp.usercontrols;

import java.util.Collection;
import java.util.Iterator;
import java.util.Vector;

import ca.uvic.csr.shrimp.SelectorBeanConstants;
import ca.uvic.csr.shrimp.DisplayBean.DisplayConstants;
import ca.uvic.csr.shrimp.DisplayBean.ShrimpNode;
import ca.uvic.csr.shrimp.SelectorBean.SelectorBean;
import ca.uvic.csr.shrimp.ShrimpApplication.BeanNotFoundException;
import ca.uvic.csr.shrimp.ShrimpProject.ShrimpProject;
import ca.uvic.csr.shrimp.ShrimpTool.ShrimpTool;

/**
 * This adapter selects the nodes connected by incoming or outgoing arcs or both of the currently selected nodes.
 * 
 * @author Jeff Michaud, Rob Lintern, Chris Callendar
 */
public class SelectConnectedNodesAdapter extends ConnectedNodesAdapter {
	
    /**
     * @param tool
     * @param direction see {@link DisplayConstants}
     * @see DisplayConstants#INCOMING_AND_OUTGOING
     * @see DisplayConstants#INCOMING
     * @see DisplayConstants#OUTGOING
     */
    public SelectConnectedNodesAdapter(ShrimpProject project, ShrimpTool tool, String direction) {
		super("Select " + direction + " Nodes", project, tool, direction);
	}

	/** 
	 * @see ca.uvic.csr.shrimp.usercontrols.UserAction#startAction()
	 */
	public void startAction() {	
		try {
			SelectorBean selectorBean = (SelectorBean) tool.getBean(ShrimpTool.SELECTOR_BEAN);
			Vector selectedNodes = (Vector) selectorBean.getSelected(SelectorBeanConstants.SELECTED_NODES);
			Collection connectedNodes = getConnectedNodes(false, true);
			for (Iterator iter = connectedNodes.iterator(); iter.hasNext();) {
				ShrimpNode node = (ShrimpNode) iter.next();
				if (!selectedNodes.contains(node)) {
					selectedNodes.add(node);
				}
			}
			selectorBean.setSelected(SelectorBeanConstants.SELECTED_NODES, selectedNodes);
		} catch (BeanNotFoundException e) {
			e.printStackTrace();
		}
	}

}
